#ifndef _PKTIO_H_
#define _PKTIO_H_

struct pktPort;

#ifdef __cplusplus
extern "C" {
#endif

/*
 * You should acquire the read semaphore before
 * reading and release it only when really done.
 * In most cases the same thread will always be reading
 * the port and so will hold the semaphore until program
 * exit.
 *
 * pktWrite() manages the semaphore for you.
 */

void pktAcquireReadSem(struct pktPort *pkt);
void pktReleaseReadSem(struct pktPort *pkt);

/*
 * pktRead() returns 0 if no
 * packet or size of packet
 */

int
pktRead(struct pktPort *pkt,
	unsigned char *data,
	unsigned char *port,
	unsigned char *id,
	unsigned char *opcode);

/*
 * pktWrite() returns -1 if the write
 * failed or size of packet.
 */

int
pktWrite(struct pktPort *pkt,
	 unsigned char port,
	 unsigned char id,
	 unsigned char opcode,
	 unsigned char *data,
	 int size);

/*
 * pktPort() returns a pktPort handle
 * used when calling pktRead() and pktWrite()
 */

int pktGetFd(struct pktPort *pkt);
struct pktPort *pktInitPort(int fd);

/*
 * pktAddxx() updates pos to point to the next data.
 * It might be a good idea to make these inline functions.
 */

/* These really belong someplace else */

void pktAddChar  (unsigned char *buffer, int *pos, unsigned char data);
void pktAddShort (unsigned char *buffer, int *pos, unsigned short data);
void pktAddLong  (unsigned char *buffer, int *pos, unsigned long data);
void pktAdd (unsigned char *buffer, int *pos, unsigned char *data, int nAdd);

unsigned char  pktGetChar  (unsigned char *buffer, int *pos);
unsigned short pktGetShort (unsigned char *buffer, int *pos);
unsigned long  pktGetLong  (unsigned char *buffer, int *pos);
void pktGet (unsigned char *buffer, int *pos, unsigned char *data, int nGet);

#ifdef __cplusplus
}
#endif

#endif
