#!/usr/bin/perl

# Kris Beevers (beevek@cs.rpi.edu)

# Sharp GP2D02 IR calibration for MAGE.  This only works for the
# version of MAGE that we have modified ourselves (which is on the
# Magellan in root's home directory as Mage-0.31).

# Switch to the IR console on the Magellan.  The IR sensors are
# controlled by two boards; you can switch between them by changing
# the box labeled "0" or "1".  Board 0 controls the sensors on the
# back panel of the robot.  The leftmost value is the IR next to the
# latch, the rightmost is the one next to the hinge.  The same is true
# for board 1, which controls the front panel.

# Hold a blank piece of white paper 20cm away from each sensor and
# record the value on the IR console for the sensor.  Do the same at
# 60cm from each sensor.  Put these values in the following two
# arrays.  The value at index 0 is the frontmost sensor, and the
# arrays go COUNTERCLOCKWISE around the radius of the robot.

@raw_20cm = (0x75, 0x74, 0x9A, 0x91, 0x85, 0x75, 0xAE, 0xA9, 0x6A,
	     0x75, 0x82, 0x97, 0x77, 0xA0, 0x98, 0x93);

@raw_60cm = (0x43, 0x43, 0x65, 0x64, 0x53, 0x47, 0x62, 0x78, 0x39,
	     0x4E, 0x50, 0x68, 0x44, 0x6D, 0x66, 0x5F);

# These values are used to compute Kg (gain) and K0 (offset) in the
# model:
#   d(x) = Kg/(x-K0)
# which converts a raw sensor value (x) to a distance in meters.  This
# model is from http://barello.net/Papers/GP2D02.  Given the raw data
# for a sensor, Kg and K0 are computed as:
#   K0 = (x60*0.6 - x20*0.2) / (0.6-0.2)
#   Kg = 0.6*(x60-K0)

# The output is C code which should be placed in the file
# include/irCalibration.h in the MAGE distribution.  Then, MAGE should
# be recompiled and reinstalled on the Magellan.  (gmake clean; gmake;
# gmake install)

for($i = 0; $i < 16; ++$i) {
    $K0[$i] = (0.6*$raw_60cm[$i] - 0.2*$raw_20cm[$i]) / (0.6-0.2);
    $Kg[$i] = 0.6*($raw_60cm[$i] - $K0[$i]);
}

print "float ir_K0[] = {";
for($i = 0; $i < 15; ++$i) {
    print "$K0[$i], ";
}
print "$K0[15]};\n";
print "float ir_Kg[] = {";
for($i = 0; $i < 15; ++$i) {
    print "$Kg[$i], ";
}
print "$Kg[15]};\n";
