/*
 * Mage.c
 * Generic low-level access to the MagellanPro
 * Copyright 2000, 2001 Swarthmore College
 */

/* defines */

#define MAX(x, y) (( (x) > (y) ) ? (x) : (y))
#define MIN(x, y) (( (x) < (y) ) ? (x) : (y))

/* includes */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <sys/stat.h>
#include <ctype.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>          /* for "bind" system calls */
#include <sys/time.h>           /* for "select" system call */
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <pthread.h>
#include <math.h>
#include "Mage.h"
#include "rFlex.h"
#include "irCalibration.h"
#include "serial.h"


/********************
 *                  *
 * Global Variables *
 *                  *
 ********************/

struct reflex *reflex;
long* State;

int sonarTrans[24] = {11, 10, 9, 8, 7, 6, 5, 4, 
		      -1, -1, -1, -1, -1, -1, -1, -1, 
		      3, 2, 1, 0, 15, 14, 13, 12}; 
int irTrans[16] = {11, 10, 9, 8, 7, 6, 5, 4, 
		   3, 2, 1, 0, 15, 14, 13, 12};

int motionStale = 1;
int axis0Stale = 1;
int axis1Stale = 1;

// Odometry globals

long xPos = 0;
long yPos = 0;
long theta = 0;

long startAxis0 = 0;
long startAxis1 = 0;

long axis0 = 0;
long axis1 = 0;

long origin0 = 0;

/* ============================================================ */
// Callback functions:     
/* ============================================================ */

static int
sysStatusRpl(struct reflex *reflex, int rv, unsigned char id, int ts,
	     int volts, unsigned char brake)
{
  //  voltage = volts;
  return(0);  
}

/* ============================================================ */
/* ============================================================ */

static int
motAxisGetSystemRpl (struct reflex *reflex, int rv,
		     unsigned char id, int ts,
		     unsigned char axis,
		     int pos, int vel, int acc, int trq)
{  
  if (axis == 0) {
    // State[STATE_VEL_TRANS] = vel/32;
    axis0 = pos;
    axis0Stale = 0;
  }
  else if (axis == 1) {
    // State[STATE_VEL_STEER] = vel/32;
    axis1 = pos;
    axis1Stale = 0;
  }
  
  return(0);
}

/* ============================================================ */
/* ============================================================ */

int
sonarGetUpdateRpl (struct reflex *reflex, int rv, unsigned char id, int ts,
                   int count, unsigned char *sonarId,
                   unsigned short *sonarValue)
{
  int ii;
  for (ii=0; ii<count; ii++) {
    State[STATE_SONAR_0+sonarTrans[sonarId[ii]]]=
      MIN(8000,sonarValue[ii]/1.415);
  }
  return(0);
}

/* ============================================================ */
/* ============================================================ */

int
irRpt (struct reflex *reflex, unsigned char id, int ts[2],
       unsigned char *flag,
       unsigned short *irValue)
{
  int ii;
  // long temp;
  for (ii=0; ii<16; ii++) {
    /***** ORIGINAL MAGE CODE *****
    // temp= irValue[ii]*(40.0/irCalib[ii]); // calibration
    // fprintf(stderr,"irVal:%d = %d, irCalib:%d = %d\n",ii,temp,ii,irCalib[ii]); 
    State[STATE_IR_0+irTrans[ii]]=10 * 9500*pow(irValue[ii],-1.3); // transform of the non-linear sensor range
    */

    // KRIS MODIFIED THIS CODE FROM THE ORIGINAL MAGE
    State[STATE_IR_0+irTrans[ii]] = (long)(1000.0 * ir_Kg[ii] / (irValue[ii] - ir_K0[ii]));
    if(State[STATE_IR_0+irTrans[ii]] < 0)    // a garbage value (nothing in range)
      State[STATE_IR_0+irTrans[ii]] = 10000; // way bigger than max range

  }
  return(0);
}

/* ============================================================ */
/* ============================================================ */

int
dioUpdate (struct reflex *reflex, unsigned char id, int ts,
	   unsigned char netId, unsigned char offset, unsigned char state)
{
  return(0);
}

/* ------------------------------------------------------------ */

int
dioRpt (struct reflex *reflex, unsigned char id, int ts,
	   unsigned char netId, unsigned char offset, unsigned char state)
{
  int i;
  
  State[STATE_BUMPER] = 0;
  
  if (netId == 64) { 
    // fprintf(stderr,"st64: %i\n",(int)state);
    for (i=0;i<8;i++) {
      if ((((int) state)>>i) == 1) State[STATE_BUMPER] |= (1<<(11-i));
    }
  }
  
  if (netId == 65) { 
    // fprintf(stderr,"st65: %i\n",(int)state);
    for (i=0;i<4;i++) {
      if ((((int) state)>>i) == 1) State[STATE_BUMPER] |= (1<<(3-i));
    }
    for (i=4;i<8;i++) {
      if ((((int) state)>>i) == 1) State[STATE_BUMPER] |= (1<<(19-i));
    }
  }
  //  fprintf(stderr,"bump: %ld\n",State[STATE_BUMPER]);
  return(0);
}


/* ============================================================ */
/* ============================================================ */

int updatePos ()
{
  long tempTheta = 0;
  long aveTheta = 0;
  static long revs = 29700; // 32000;
  static float fact = 4.727; //5.09; // (revs/(2*3.1415)/1000); 
  
  if (State[STATE_JUMP] != 0) {
    xPos=(State[STATE_JUMPX]<<5);
    yPos=(State[STATE_JUMPY]<<5);
    theta=(State[STATE_JUMPT]*fact);
    // fprintf(stderr,"Jumped\n"); 
    State[STATE_JUMP] = 0;
  }

  // if ((axis1Stale==1) || (axis0Stale==1)) return(1);
  tempTheta = (axis1-startAxis1)%revs;
  aveTheta = theta + (tempTheta>>1);
  xPos+=cos(6.28*(aveTheta%revs)/revs)*(axis0-startAxis0);
  yPos+=sin(6.28*(aveTheta%revs)/revs)*(axis0-startAxis0);
  theta = (theta + tempTheta) %revs;

  State[STATE_TRANS] = (axis0-origin0)/32;
  State[STATE_X] = xPos/32;
  State[STATE_Y] = yPos/32;

  State[STATE_T] = theta/fact; // (theta/306)*(2*pi))
  //  fprintf(stderr,"axis1-startAxis1= %ld\n",axis1-startAxis1);  
  
  // fprintf(stderr,"axis1:%ld\n",axis1);

  if (State[STATE_T] < 0) State[STATE_T]+=6283;
  
  startAxis0 = axis0;
  startAxis1 = axis1;
  axis0Stale = 1;
  axis1Stale = 1;
  
  return (0);
}

/* ============================================================ */
/* ============================================================ */

int updateMotorCmd(){
  int t_dir;
  int s_dir;
 
  t_dir = (State[STATE_VEL_TRANS]<0 ? 0 : 1);
  s_dir = (State[STATE_VEL_STEER]<0 ? 0 : 1);
  
  reflex->motion->setDirReq(reflex, 0,
			    1,                         // rotation axis 
			    abs(State[STATE_VEL_STEER]*306/36), // velocity
			    State[STATE_ACC_STEER]*306/36,      // acceleration
			    5000,                      // torque
			    s_dir);                    // direction
  reflex->motion->setDirReq(reflex, 0, 
			    0,                         // translation axis
			    abs(State[STATE_VEL_TRANS]*32),   // velocity
			    State[STATE_ACC_TRANS]*32,        // acceleration
			    5000,                      // torque
			    t_dir);                    // direction
 
  return 0;
}


void *watchBot (void *arg)
{
  int i; 
  while (1) {
    for(i=0;i<10;i++) {
      usleep(10000);
      reflex->read(reflex, 5); 
      updatePos();
    } 
    updateMotorCmd();
    reflex->motion->getSystemReq(reflex,255, 0); // trans axis
    reflex->motion->getSystemReq(reflex,255, 1); // rotation axis
    reflex->sonar->updateReq(reflex,255,0); 
  }
}

/* ============================================================ */
/* ============================================================ */

void connectRobot(long* St)
{
  pthread_t *watcher;
  int fd; 

  State = St;
  State[STATE_BUMPER] = 0;

  fprintf(stderr, "Mage 0.2\n");
    
  fd = openSerialPort("/dev/ttyS1",115200);
  
  reflex = reflexNew(fd, NULL);

  reflex->system->statusRpl       = sysStatusRpl;
  reflex->motion->getSystemRpl    = motAxisGetSystemRpl;
  reflex->sonar->updateRpl        = sonarGetUpdateRpl;
  reflex->ir->rpt                 = irRpt;
  reflex->dio->update             = dioUpdate;
  reflex->dio->rpt                = dioRpt;

  reflex->motion->setDefaultsReq(reflex, 255);  // init the motors
  reflex->dio->rptReq(reflex, 0, 10000); // turn on bump sensors

  reflex->motion->getSystemReq(reflex, 255, 0);  // translation axis
  reflex->motion->getSystemReq(reflex, 255, 1);  // rotation axis
  
  axis0Stale = axis1Stale = 1;

  while ((axis0Stale==1) || (axis1Stale==1)){ // wait for bot to report its position
    reflex->read(reflex, 5);
    usleep(10000);
  }

  zr();
  
  watcher = malloc(sizeof(pthread_t));
  pthread_create(watcher,NULL,&watchBot,&reflex);
  
}

int disconnectRobot(void)
{
  irOff();
  sonarOff();
  vm(0,0);
  
  return 0;
}

/******************************************************/
int vm(
       long t_vel, // millimeters/sec
       long s_vel) // milliradians/sec
{
  State[STATE_ACC_TRANS] = 3000;
  State[STATE_ACC_STEER] = 8000;
  State[STATE_VEL_TRANS] = t_vel;
  State[STATE_VEL_STEER] = s_vel;
  return 0;
}

int vam(
	long t_vel, // mmeters/sec
	long t_acc, // mmeters/sec/sec
	long s_vel, // mradians/sec
	long s_acc) // mradians/sec/sec
{
  State[STATE_ACC_TRANS] = t_acc;
  State[STATE_ACC_STEER] = s_acc;
  State[STATE_VEL_TRANS] = t_vel;
  State[STATE_VEL_STEER] = s_vel;
  return 0;
}

int zr(void)
{
  fprintf(stderr,"zr 0:%ld 1:%ld\n",axis0,axis1);
  
  origin0 = axis0;
  startAxis0 = axis0;
  startAxis1 = axis1;
  xPos = 0;
  yPos = 0;
  theta = 0;
  return(0);
}

int jump(long x, long y, long t)
{
  xPos = x<<5;
  yPos = y<<5;
  theta = t*4.87;
  return(0);
}

int irOff(void) 
{
  reflex->ir->rptReq(reflex, 255, 
		     0,    // triggerDelay (millisec)
		     0,    // fireDelay
		     0,    // retryDelay
		     20,   // setDelay
		     0,    // goneTimeout
		     0);   // runMode (0= off, 1 = report on request, 2 = continuous reporting)
  return 0;
}

int irOn(void)
{
  reflex->ir->rptReq(reflex, 255, 
		     0,        // triggerDelay (millisec)
		     70,       // fireDelay
		     10,       // retryDelay
		     20,       // setDelay
		     150,      // goneTimeout
		     2);       // runMode (0= off, 1 = report on request, 2 = continuous reporting)
  return 0;
}

int sonarOff()
{
  reflex->sonar->rptReq(reflex, 255, 
			0,  // echoDelay
			0,  // pingDelay
			0,  // setDelay
			0); // mode (0= off, 1 = report on request, 2 = continuous reporting)
  return 0;
}

int sonarOn()
{
  reflex->sonar->rptReq(reflex, 255, 
			30000,  // echoDelay default: 30000
			0,      // pingDelay default: 0
			0,      // setDelay
			1);     // mode (0= off, 1 = report on request, 2 = continuous reporting)
  return 0;
}










