// NOTE:
// This code relies heavily on the Canon VC-C4 command set.  
//
//  NAPPOC (Not A Polished Piece Of Code)
//
//  Nathaniel Fairfield [than@sccs.swarthmore.edu]

#include <sys/ioctl.h>
#include <sys/types.h>
#include <math.h>

#include <unistd.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <string.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>
#include <sys/timeb.h>
#include <pthread.h>
#include <serial.h>
#include <ptzClient.h>
#include <robotVision.h>

unsigned char cmdbuf[32];
unsigned char newbuf[32];
int cmdLength;
int fd;

#define MIN(x, y) (( (x) < (y) ) ? (x) : (y))
#define MAX(x, y) (( (x) > (y) ) ? (x) : (y))

#define MAXPAN 1778
#define MINPAN -1779
#define MAXTILT 3172
#define MINTILT 1037


// globals
short pan;
short tilt;
short panout;
short tiltout;
short zoom;
short ptzStale = 1;
short trackState = 0; // 0 setting, 1 tracking, 2 lost

/***********************
    Utility functions 
 ***********************/

int readyPTZ () 
{
  if (cmdLength == 0) return 1;
  else return 0;
}

int donePTZ ()
{
  if ((pan == panout) && (tilt == tiltout)) return 1;
  else return 0;
}

void hexPrint(unsigned char buf[], unsigned char trm)
{
  int ii = 0;
  fprintf(stderr,"hexPrint:  ");
  while (buf[ii] != trm) fprintf(stderr," %02X",buf[ii++]);
  fprintf(stderr," %02X",buf[ii]);
  fprintf(stderr,"\n");
}

/***********************
  PTZ Command functions
 ***********************/

void homePTZ()
{
  cmdLength = 6;
  sprintf(cmdbuf,"%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0x57,0xEF);
}

// very simple zoom state functionality
int zoomState = 0;

int getZoom(void) {
  return(zoomState);
}

#define HFOVCalibrationEq(a) ((a) <= 65 ? 93.0 - 0.85 * (float)(a) :  67.0 - 0.473 * (float)(a))
#define VFOVCalibrationEq(a) ((a) <= 65 ? 63.0 - 0.62 * (float)(a) :  42.0 - 0.31 * (float)(a))

long getHFOV(void) { // returns the current field of view of the camera in mrad
  static int lastZoom = 0;
  static long lastValue = VISION_DEFAULT_HFOV;
  float cmAt1m;
  float rad;
  
  if(lastZoom == zoomState)
    return(lastValue);

  cmAt1m = HFOVCalibrationEq(zoomState);
  rad = atan2(cmAt1m/2.0, 100.0);

  lastZoom = zoomState;
  lastValue = (long)(rad * 2000.0);

  return(lastValue);
}

long getVFOV(void) { // returns the current field of view of the camera in mrad
  static int lastZoom = 0;
  static long lastValue = VISION_DEFAULT_VFOV;
  float cmAt1m;
  float rad;
  
  if(lastZoom == zoomState)
    return(lastValue);

  cmAt1m = VFOVCalibrationEq(zoomState);
  rad = atan2(cmAt1m/2.0, 100.0);

  lastZoom = zoomState;
  lastValue = (long)(rad * 2000.0);

  return(lastValue);
}

long lookupHFOV(int zoom) {
  double cmAt1m;
  double rad;

  cmAt1m = HFOVCalibrationEq(zoom);
  rad = atan2(cmAt1m/2.0, 1000);

  return((long)(rad * 2000.0));
}

long lookupVFOV(int zoom) {
  double cmAt1m;
  double rad;

  cmAt1m = VFOVCalibrationEq(zoom);
  rad = atan2(cmAt1m/2.0, 1000);

  return((long)(rad * 2000.0));
}
 
void setZoom(int zoom)  // value between 0 (wide)  and 128 (tele) in decimal
{
  short zoomvalue;
  char digit1, digit2;

  // bound it
  zoomvalue = MIN(128, zoom);
  zoomvalue = MAX(0, zoomvalue);

  // Now we have to convert it to an ascii representation of hexadecimal
  digit1 = zoomvalue / 16 + 0x30;
  digit2 = zoomvalue % 16 + 0x30;

  // fix the A-F characters
  digit1 = digit1 > 0x39 ? (digit1 - 0x39) + 0x40 : digit1;
  digit2 = digit2 > 0x39 ? (digit2 - 0x39) + 0x40 : digit2;

  // fill out the command
  sprintf(cmdbuf, "%c%c%c%c%c%c%c%c",0xFF, 0x30, 0x30, 0x00, 0xA3, digit1, digit2, 0xEF);
  cmdLength = 8;

  zoomState = zoomvalue;
}

void setPanTilt(short panin, short tiltin)
{
  short tpan, ttilt;

  // these are empricially tested min and max values on the canon
  panin = MIN(1778, panin);
  panin = MAX(-1779, panin);
  tiltin = MIN(3172, tiltin);
  tiltin = MAX(1037, tiltin);

  // transformation to a uniform space (0 => down, PI => up)
  panin = 6282 - panin;
  tiltin-=1571;

  if (panin > 3141) panin-=6282;
  if (tiltin > 3141) tiltin-=6282;

  pan = (int) (panin>>1);
  tilt = (int) (tiltin>>1);

  tpan = pan;
  ttilt = tilt;

  // these bounds checks are not effective
  tpan = MIN(890,tpan);
  tpan = MAX(-890,tpan);
  ttilt = MIN(800,ttilt);
  ttilt = MAX(-267,ttilt);

  // adjustment to Canon coordinate system
  tpan = 32768 + tpan;
  ttilt = 32768 + ttilt;
  cmdLength = 14;
  sprintf(cmdbuf,"%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
	  0xFF,0x30,0x30,0x00,0x62,
	  ((tpan >> 12) & 0x0F)+((((tpan >> 12) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((tpan >> 8 ) & 0x0F)+((((tpan >> 8 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((tpan >> 4 ) & 0x0F)+((((tpan >> 4 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((tpan >> 0 ) & 0x0F)+((((tpan >> 0 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 12) & 0x0F)+((((ttilt >> 12) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 8 ) & 0x0F)+((((ttilt >> 8 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 4 ) & 0x0F)+((((ttilt >> 4 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 0 ) & 0x0F)+((((ttilt >> 0 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  0xEF
	  );

}

void initPTZ ()
{
  cmdLength = 7;
  sprintf(cmdbuf,"%c%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0x90,0x30,0xEF);
  while(!readyPTZ());

  cmdLength = 7;
  sprintf(cmdbuf,"%c%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0xA5,0x33,0xEF);
  while(!readyPTZ());
  /*
  cmdLength = 7;
   sprintf(cmdbuf,"%c%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0xA7,0x31,0xEF);
   while(!readyPTZ());
  */
}

void getPTZ (int pt[3])
{
  pt[0] = - ((int) panout<<1);
  pt[1] = 1571 + ((int) tiltout<<1);
  pt[2] = (int) zoomState;
}

void *watchPTZ (void *arg)
{
  int cmdLengthTemp = 0;
  unsigned char tmpbuf[128];
  unsigned char msgbuf[128];
  while (1) {
    // fprintf(stderr,".");
    usleep(100000);
      
    if (cmdLength > 0) {
      // fprintf(stderr,"Sending ");
      // hexPrint(cmdbuf,0xEF);
      write(fd,cmdbuf,cmdLength);
      cmdLengthTemp = cmdLength;
      cmdLength = 0;
    } 
    else {
      // if no command, get cameraposition...
      sprintf(tmpbuf,"%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0x63,0xEF);
      write(fd,tmpbuf,6);
    }

    while (1) { 
      usleep(40000);
	  
      sprintf(msgbuf," ");
      read(fd,msgbuf,255);
	  
      if (msgbuf[0] == 0xFE) {
	// fprintf(stderr,"Got ");
	// hexPrint(msgbuf,0xEF);
	      
	if ((msgbuf[1] == 0x30)
	    && (msgbuf[2] == 0x30)
	    && (msgbuf[3] == 0x30)
	    && (msgbuf[4] == 0x30))
	  {
	    if (msgbuf[5] != 0xEF) {
		    
	      panout = (((msgbuf[5]&0x0F)+((msgbuf[5]>0x40)?(0x09):(0)))<<8)
		+ (((msgbuf[6]&0x0F)+((msgbuf[6]>0x40)?(0x09):(0)))<<8) 
		+ (((msgbuf[7]&0x0F)+((msgbuf[7]>0x40)?(0x09):(0)))<<4) 
		+ ((msgbuf[8]&0x0F)+((msgbuf[8]>0x40)?(0x09):(0)));
	      tiltout = (((msgbuf[9]&0x0F)+((msgbuf[9]>0x40)?(0x09):(0)))<<8)
		+ (((msgbuf[10]&0x0F)+((msgbuf[10]>0x40)?(0x09):(0)))<<8) 
		+ (((msgbuf[11]&0x0F)+((msgbuf[11]>0x40)?(0x09):(0)))<<4) 
		+ ((msgbuf[12]&0x0F)+((msgbuf[12]>0x40)?(0x09):(0)));
	      panout -= 2048;
	      tiltout -= 2048;
		    
	      if (panout > 2048) panout-=3840;
	      if (tiltout > 2048) tiltout-=3840;
	    }
	  }
	else { // error!!! 
	  fprintf(stderr,"PTZ error:  resending command.  Error ");
	  hexPrint(msgbuf,0xEF);
	  if (cmdLength == 0) cmdLength = cmdLengthTemp;
	  break;
	}
	break;
      }
    }
  } 
}

int startPTZ()
{
  pthread_t *watcher;
  cmdLength = 0;
  cmdbuf[0] = 0x01;
  
  fd = openSerialPort("/dev/ttyS3", 9600);
  
  watcher = malloc(sizeof(pthread_t)); 
  pthread_create(watcher,NULL,&watchPTZ,NULL);
  
  while (!readyPTZ());
  initPTZ();
  while (!readyPTZ());

  return 0;
}
