#include <unistd.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <signal.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <termios.h>
#include "ptzClient.h"
#include "Mage.h"
#include "MageUtil.h"

typedef void (*sighandler_t)(int);
static struct termios save_termios;
static int ttysavefd = -1;
static enum {RESET, CBREAK} ttystate = RESET;
int tty_reset(int fd);

long State[NUM_STATE];

/*------------------------------------------------------------*/

int tty_cbreak(int fd) { /* put terminal in cbreak mode */
  struct termios buf;
  
  if (tcgetattr(fd, &save_termios) < 0)
    return (-1);

  buf=save_termios;
  buf.c_lflag &= ~(ECHO | ICANON); /* echo off, cononacal mode off */
  buf.c_cc[VMIN] = 0; /* Case B: 1 byte at a time, no timer */
  buf.c_cc[VTIME] = 0;

  if (tcsetattr(fd, TCSAFLUSH, &buf) < 0)
    return (-1);
  ttystate = CBREAK;
  ttysavefd=fd;
  return (0);
}

/*------------------------------------------------------------*/

int tty_reset(int fd) { /* restore terminal's mode */
  if (ttystate != CBREAK)
    return (0);

  if (tcsetattr(fd, TCSAFLUSH, &save_termios) <0)
    return (-1);
  ttystate=RESET;
  return (0);
}

/*------------------------------------------------------------*/

void
ttyExit(void)
{
  fprintf(stderr, "Resetting tty mode... ");
  
  if (tty_reset(STDIN_FILENO)<0) {
    fprintf(stderr, "tty_reset() error\n");
  }
  else {
    fprintf(stderr, "done.\n");
  }
  
  fprintf(stderr, "Exiting.\n");

  exit(0);
}

void sighandler(int signal) {
  // for all signals, detach and terminate the shared memory connection
  irOff();
  disconnectRobot();
  ttyExit();
  printf("Caught signal %d, exiting.\n", signal);

  exit(-1);
}


int nav (void) {
  int trans, steering;
  int ii,size;
  char buffer[255];
  fd_set fds;
  struct timeval timeout;
  int retval;
  int go = 1;
  int print = 0;
  int speed = 200;

  
  timeout.tv_sec = 1;
  timeout.tv_usec = 0;
  
  while (go) { 
    if (print == 1) fprintf(stderr,"x: %ld, y:%ld, t:%ld\n",State[STATE_X],State[STATE_Y], State[STATE_T]);

    FD_ZERO(&fds);
    FD_SET(STDIN_FILENO, &fds);
   
    usleep(200000);
    trans = 0;
    steering = 0;

    retval = select(STDIN_FILENO+1, &fds, NULL, NULL, &timeout);

    if (retval) {
      size = read(STDIN_FILENO, buffer, 255);
      for (ii=0; ii<size; ii++) {
	if (buffer[ii] == 'w' || buffer[ii] == 'i') trans = speed;
	if (buffer[ii] == 'a' || buffer[ii] == 'j') steering = speed;
	if (buffer[ii] == 's' || buffer[ii] == 'k') trans = -speed;
	if (buffer[ii] == 'd' || buffer[ii] == 'l') steering = -speed;
	if (buffer[ii] == 'q') go = 0;
	if (buffer[ii] == 'p') print = (print == 1)?0:1;
	if (buffer[ii] == '1') speed = 100;
	if (buffer[ii] == '2') speed = 200;
	if (buffer[ii] == '3') speed = 300;
	if (buffer[ii] == '4') speed = 400;
	    }
    }
    if (State[STATE_BUMPER] > 0){
      vm(0,0);
      continue;
    }
    vm(trans,steering);
    // usleep(500000);
  }
  return 1;
}


int main(int argc, char *argv[])
{
  // set up signal handlers
  signal(SIGINT, (sighandler_t)sighandler);
  signal(SIGQUIT, (sighandler_t)sighandler);
  signal(SIGTERM, (sighandler_t)sighandler);
  signal(SIGHUP, (sighandler_t)sighandler);
  signal(SIGFPE, (sighandler_t)sighandler);

  if (tty_cbreak(STDIN_FILENO) < 0) {
    fprintf(stderr, "tty_cbreak() error\n");
    exit (0);
  }


  connectRobot(State);
  
  printf("A small utility for driving the robot around using the keyboard.\n");
  printf("w/i = forward, s/k = back, a/j = left, d/l = right, q to quit. p to print position\n");
  
  nav();

  disconnectRobot();
  ttyExit();
  return 0;
}




