#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */
/* ============================================================ */

static int
stateReq(struct reflex *reflex, unsigned char id)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = JSTK_GET_STATE;
  port   = reflex->joystick->port;

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}

/*------------------------------------------------------------*/

static int
stateRpl(struct reflex *reflex, unsigned char id,
	 int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int ts, x, y;
  unsigned char buttons;
  int rv = 0;

  /* should check on len and opcode */
  
  ii = 0;

  ts      = pktGetLong(buffer, &ii);
  x       = pktGetLong(buffer, &ii);
  y       = pktGetLong(buffer, &ii);
  buttons = pktGetChar(buffer, &ii);
  
  if (reflex->joystick->stateRpl) {
    retval = reflex->joystick->stateRpl(reflex, rv, id, ts, x, y, buttons);
  }
  return(retval);
}

/* ============================================================ */
/* ============================================================ */
/* ============================================================ */

static int
handlePort(struct reflex *reflex, int size, unsigned char *buffer,
	   unsigned char id, unsigned char opcode)
{
  int retval;

  switch (opcode) {
  case JSTK_GET_STATE:
    retval = stateRpl(reflex, id, size, buffer);
    break;

  default:
    if (reflex->joystick->unknownRpl) {
      retval = reflex->joystick->unknownRpl(reflex, id, size, buffer, opcode);
    }
  }
  
  return(retval);
}
  
/* ============================================================ */
/* ============================================================ */
  
static int
destroy (struct reflexJoystick *ptr)
{

  if (ptr) free(ptr);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflexJoystick *
reflexJoystickNew (int port)
{
  struct reflexJoystick *joystick;

  joystick = malloc(sizeof(*joystick));
  if (!joystick) return(NULL);
  memset(joystick, 0, sizeof(*joystick));

  joystick->port       = port;

  joystick->stateReq   = stateReq;

  joystick->handlePort = handlePort;
  joystick->destroy    = destroy;

  return(joystick);
}
